@@ -816,6 +816,7 @@ from .ninecninemedia import NineCNineMediaIE
 from .ninegag import NineGagIE
 from .ninenow import NineNowIE
 from .nintendo import NintendoIE
+from .nitter import NitterSHIE
 from .njpwworld import NJPWWorldIE
 from .nobelprize import NobelPrizeIE
 from .noco import NocoIE
@@ -912,7 +913,12 @@ from .parliamentliveuk import ParliamentLiveUKIE
 from .patreon import PatreonIE
 from .pbs import PBSIE
 from .pearvideo import PearVideoIE
-from .peertube import PeerTubeSHIE
+from .peertube import (
+    PeerTubeSHIE,
+    PeerTubePlaylistSHIE,
+    PeerTubeAccountSHIE,
+    PeerTubeChannelSHIE,
+)
 from .people import PeopleIE
 from .performgroup import PerformGroupIE
 from .periscope import (
